require "qcs/version"
require 'json'
require 'optparse'
require 'colorize'

class RepoInfo
  attr_accessor :name, :url, :branch

  def initialize(name, url, branch)
    @name = name
    @url = url
    @branch = branch
  end
end

class QCS
  attr_accessor :repos, :mainPro

  def initialize(filePath)
    fileText = IO.read(filePath)
    @repos = Array.new
    JSON.parse(fileText).each do |repo|
      name = repo["name"]
      url = repo["url"]
      branch = repo["branch"]
      parsed = RepoInfo.new(name, url, branch)
      @repos.push(parsed)
      if repo["isMain"] == 1
        @mainPro = parsed
      end
    end
  end

  def pull
    @repos.each do |repoInfo|
      unless Dir.exist?(repoInfo.name)
        clone(repoInfo)
        return
      end

      Dir.chdir(repoInfo.name) do
        hint = "pull out #{repoInfo.name} ..."
        puts(hint.green)
        result = `git pull`
        puts result
      end
    end
  end

  def install
    @repos.each do |repoInfo|
      clone repoInfo
    end
    Dir.chdir(@mainPro.name) do
      puts "pod install ... #{Dir.pwd}"
      result = `pod install`
      puts result
    end
  end

  def clone(repoInfo)
    unless Dir.exist?(repoInfo.name)
      hint = "clone #{repoInfo.name} ..."
      puts hint
      result = `git clone -b #{repoInfo.branch} #{repoInfo.url} #{repoInfo.name}`
      puts result
    end
  end

  def checkout(branch)
    @repos.each do |repoInfo|
      Dir.chdir(repoInfo.name) do
        hint = "check out #{repoInfo.name} ..."
        puts(hint.green)
        result = `git checkout #{branch}`
        puts result
      end
    end
  end
end

def main(args)
  opt_parser = OptionParser.new do |opt|
    opt.banner = "Usage: #{"qcs COMMAND".green} [#{"OPTIONS".green}]"
    opt.separator "    "
    opt.separator "Commands"
    opt.separator "    "
    opt.separator "     #{"+ pull".green} Fetch from and integrate with another repository or a local branch"
    opt.separator "     #{"+ checkout".green} Switch branches or restore working tree files"
    opt.separator "    "
    opt.separator "Options"
    opt.separator "    "
    opt.on_tail("-h", "--help", "help") do
      puts opt_parser
    end
    opt.on_tail("-v", "--version", "Show the version of the tool") do
      puts Qcs::VERSION
    end
  end

  if args.empty?
    puts opt_parser
    exit
  end

  validOpts = opt_parser.candidate(args[0]) # 校验合法的options
  unless validOpts.empty?   # 处理有options的情况 直接处理
    opt_parser.parse!(args)
    exit
  end

  config = "repos.json"
  if File.exist?(config) # 文件存在 先校验action
    qcs = QCS.new(config)
    action = args[0]
    case action
    when "install"
      qcs.install
    when "pull"
      qcs.pull
    when "checkout"
      if args.count < 2
        hint = "Invalid branch or commitId"
        STDERR.puts(hint.red)
        exit
      end
      branch = args[1]
      qcs.checkout(branch)
    else # 不合法的action
      puts opt_parser
    end
  else
    # 文件不存在 options还不合法 可能是输入了错误的命令或者options
    puts "make sure current directory has a file named #{config}"
  end
end


